@@ -11,6 +11,9 @@ def get_commands_help():
     """Generate commands help using Rich Text objects to avoid markup conflicts."""
     from rich.text import Text
 
+    # Ensure plugins are loaded so custom help can register
+    _ensure_plugins_loaded()
+
     # Build help text programmatically
     help_lines = []
 
@@ -90,6 +93,33 @@ def get_commands_help():
         + Text("            Show unknown command warning")
     )
 
+    # Add custom commands from plugins (if any)
+    try:
+        from code_puppy import callbacks
+
+        custom_help_results = callbacks.on_custom_command_help()
+        # Flatten various returns into a list of (name, description)
+        custom_entries = []
+        for res in custom_help_results:
+            if not res:
+                continue
+            if isinstance(res, tuple) and len(res) == 2:
+                custom_entries.append(res)
+            elif isinstance(res, list):
+                for item in res:
+                    if isinstance(item, tuple) and len(item) == 2:
+                        custom_entries.append(item)
+        if custom_entries:
+            help_lines.append(Text("\n", style="dim"))
+            help_lines.append(Text("Custom Commands", style="bold magenta"))
+            for name, desc in custom_entries:
+                help_lines.append(
+                    Text(f"/{name}", style="cyan") + Text(f"  {desc}")
+                )
+    except Exception:
+        # If callbacks fail, skip custom help silently
+        pass
+
     # Combine all lines
     final_text = Text()
     for i, line in enumerate(help_lines):
@@ -100,8 +130,32 @@ def get_commands_help():
     return final_text
 
 
+_PLUGINS_LOADED = False
+
+
+def _ensure_plugins_loaded() -> None:
+    global _PLUGINS_LOADED
+    if _PLUGINS_LOADED:
+        return
+    try:
+        from code_puppy import plugins
+
+        plugins.load_plugin_callbacks()
+        _PLUGINS_LOADED = True
+    except Exception as e:
+        # If plugins fail to load, continue gracefully but note it
+        try:
+            from code_puppy.messaging import emit_warning
+
+            emit_warning(f"Plugin load error: {e}")
+        except Exception:
+            pass
+        _PLUGINS_LOADED = True
+
+
 def handle_command(command: str):
     from code_puppy.messaging import emit_error, emit_info, emit_success, emit_warning
+    _ensure_plugins_loaded()
 
     """
     Handle commands prefixed with '/'.
@@ -119,40 +173,40 @@ def handle_command(command: str):
         return True
 
     if command.strip().startswith("/compact"):
-        from code_puppy.config import get_compaction_strategy
-        from code_puppy.message_history_processor import (
-            estimate_tokens_for_message,
-            get_protected_token_count,
-            summarize_messages,
-            truncation,
-        )
+        # Functions have been moved to BaseAgent class
+        from code_puppy.agents.agent_manager import get_current_agent
+        from code_puppy.config import get_compaction_strategy, get_protected_token_count
         from code_puppy.messaging import (
             emit_error,
             emit_info,
             emit_success,
             emit_warning,
         )
-        from code_puppy.state_management import get_message_history, set_message_history
 
         try:
-            history = get_message_history()
+            agent = get_current_agent()
+            history = agent.get_message_history()
             if not history:
                 emit_warning("No history to compact yet. Ask me something first!")
                 return True
 
-            before_tokens = sum(estimate_tokens_for_message(m) for m in history)
+            current_agent = get_current_agent()
+            before_tokens = sum(
+                current_agent.estimate_tokens_for_message(m) for m in history
+            )
             compaction_strategy = get_compaction_strategy()
             protected_tokens = get_protected_token_count()
             emit_info(
                 f"🤔 Compacting {len(history)} messages using {compaction_strategy} strategy... (~{before_tokens} tokens)"
             )
 
+            current_agent = get_current_agent()
             if compaction_strategy == "truncation":
-                compacted = truncation(history, protected_tokens)
+                compacted = current_agent.truncation(history, protected_tokens)
                 summarized_messages = []  # No summarization in truncation mode
             else:
                 # Default to summarization
-                compacted, summarized_messages = summarize_messages(
+                compacted, summarized_messages = current_agent.summarize_messages(
                     history, with_protection=True
                 )
 
@@ -160,9 +214,12 @@ def handle_command(command: str):
                 emit_error("Compaction failed. History unchanged.")
                 return True
 
-            set_message_history(compacted)
+            agent.set_message_history(compacted)
 
-            after_tokens = sum(estimate_tokens_for_message(m) for m in compacted)
+            current_agent = get_current_agent()
+            after_tokens = sum(
+                current_agent.estimate_tokens_for_message(m) for m in compacted
+            )
             reduction_pct = (
                 ((before_tokens - after_tokens) / before_tokens * 100)
                 if before_tokens > 0
@@ -205,7 +262,7 @@ def handle_command(command: str):
             return True
 
     if command.strip().startswith("/show"):
-        from code_puppy.agents import get_current_agent_config
+        from code_puppy.agents import get_current_agent
         from code_puppy.command_line.model_picker_completion import get_active_model
         from code_puppy.config import (
             get_compaction_strategy,
@@ -225,7 +282,7 @@ def handle_command(command: str):
         compaction_strategy = get_compaction_strategy()
 
         # Get current agent info
-        current_agent = get_current_agent_config()
+        current_agent = get_current_agent()
 
         status_msg = f"""[bold magenta]🐶 Puppy Status[/bold magenta]
 
@@ -288,16 +345,15 @@ def handle_command(command: str):
         from code_puppy.agents import (
             get_agent_descriptions,
             get_available_agents,
-            get_current_agent_config,
+            get_current_agent,
             set_current_agent,
         )
-        from code_puppy.agents.runtime_manager import get_runtime_agent_manager
 
         tokens = command.split()
 
         if len(tokens) == 1:
             # Show current agent and available agents
-            current_agent = get_current_agent_config()
+            current_agent = get_current_agent()
             available_agents = get_available_agents()
             descriptions = get_agent_descriptions()
 
@@ -343,9 +399,9 @@ def handle_command(command: str):
 
             if set_current_agent(agent_name):
                 # Reload the agent with new configuration
-                manager = get_runtime_agent_manager()
-                manager.reload_agent()
-                new_agent = get_current_agent_config()
+                agent = get_current_agent()
+                agent.reload_code_generation_agent()
+                new_agent = get_current_agent()
                 emit_success(
                     f"Switched to agent: {new_agent.display_name}",
                     message_group=group_id,
@@ -382,13 +438,10 @@ def handle_command(command: str):
 
         new_input = update_model_in_input(model_command)
         if new_input is not None:
-            from code_puppy.agents.runtime_manager import get_runtime_agent_manager
             from code_puppy.command_line.model_picker_completion import get_active_model
 
             model = get_active_model()
             # Make sure this is called for the test
-            manager = get_runtime_agent_manager()
-            manager.reload_agent()
             emit_success(f"Active model set and loaded: {model}")
             return True
         model_names = load_model_names()
@@ -401,6 +454,8 @@ def handle_command(command: str):
 
         handler = MCPCommandHandler()
         return handler.handle_mcp_command(command)
+
+    # Built-in help
     if command in ("/help", "/h"):
         import uuid
 
@@ -427,6 +482,7 @@ def handle_command(command: str):
 
             # Get built-in agents
             from code_puppy.agents.agent_manager import get_agent_descriptions
+
             builtin_agents = get_agent_descriptions()
 
             emit_info("Available models:")
@@ -459,6 +515,7 @@ def handle_command(command: str):
 
         # Get list of available built-in agents
         from code_puppy.agents.agent_manager import get_agent_descriptions
+
         builtin_agents = get_agent_descriptions()
 
         is_json_agent = agent_name in json_agents
@@ -498,18 +555,16 @@ def handle_command(command: str):
             else:
                 # Handle built-in Python agent - store in config
                 from code_puppy.config import set_agent_pinned_model
+
                 set_agent_pinned_model(agent_name, model_name)
 
             emit_success(f"Model '{model_name}' pinned to agent '{agent_name}'")
 
             # If this is the current agent, reload it to use the new model
-            from code_puppy.agents import get_current_agent_config
-            from code_puppy.agents.runtime_manager import get_runtime_agent_manager
+            from code_puppy.agents import get_current_agent
 
-            current_agent = get_current_agent_config()
+            current_agent = get_current_agent()
             if current_agent.name == agent_name:
-                manager = get_runtime_agent_manager()
-                manager.reload_agent()
                 emit_info(f"Active agent reloaded with pinned model '{model_name}'")
 
             return True
@@ -559,9 +614,9 @@ def handle_command(command: str):
         from datetime import datetime
         from pathlib import Path
 
+        # estimate_tokens_for_message has been moved to BaseAgent class
+        from code_puppy.agents.agent_manager import get_current_agent
         from code_puppy.config import CONFIG_DIR
-        from code_puppy.message_history_processor import estimate_tokens_for_message
-        from code_puppy.state_management import get_message_history
 
         tokens = command.split()
         if len(tokens) != 2:
@@ -569,7 +624,8 @@ def handle_command(command: str):
             return True
 
         session_name = tokens[1]
-        history = get_message_history()
+        agent = get_current_agent()
+        history = agent.get_message_history()
 
         if not history:
             emit_warning("No message history to dump!")
@@ -587,11 +643,14 @@ def handle_command(command: str):
 
             # Also save metadata as JSON for readability
             meta_file = contexts_dir / f"{session_name}_meta.json"
+            current_agent = get_current_agent()
             metadata = {
                 "session_name": session_name,
                 "timestamp": datetime.now().isoformat(),
                 "message_count": len(history),
-                "total_tokens": sum(estimate_tokens_for_message(m) for m in history),
+                "total_tokens": sum(
+                    current_agent.estimate_tokens_for_message(m) for m in history
+                ),
                 "file_path": str(pickle_file),
             }
 
@@ -612,9 +671,9 @@ def handle_command(command: str):
         import pickle
         from pathlib import Path
 
+        # estimate_tokens_for_message has been moved to BaseAgent class
+        from code_puppy.agents.agent_manager import get_current_agent
         from code_puppy.config import CONFIG_DIR
-        from code_puppy.message_history_processor import estimate_tokens_for_message
-        from code_puppy.state_management import set_message_history
 
         tokens = command.split()
         if len(tokens) != 2:
@@ -638,8 +697,12 @@ def handle_command(command: str):
             with open(pickle_file, "rb") as f:
                 history = pickle.load(f)
 
-            set_message_history(history)
-            total_tokens = sum(estimate_tokens_for_message(m) for m in history)
+            agent = get_current_agent()
+            agent.set_message_history(history)
+            current_agent = get_current_agent()
+            total_tokens = sum(
+                current_agent.estimate_tokens_for_message(m) for m in history
+            )
 
             emit_success(
                 f"✅ Context loaded: {len(history)} messages ({total_tokens} tokens)\n"
@@ -652,6 +715,8 @@ def handle_command(command: str):
             return True
 
     if command.startswith("/truncate"):
+        from code_puppy.agents.agent_manager import get_current_agent
+
         tokens = command.split()
         if len(tokens) != 2:
             emit_error(
@@ -668,9 +733,8 @@ def handle_command(command: str):
             emit_error("N must be a valid integer")
             return True
 
-        from code_puppy.state_management import get_message_history, set_message_history
-
-        history = get_message_history()
+        agent = get_current_agent()
+        history = agent.get_message_history()
         if not history:
             emit_warning("No history to truncate yet. Ask me something first!")
             return True
@@ -686,7 +750,7 @@ def handle_command(command: str):
             [history[0]] + history[-(n - 1) :] if n > 1 else [history[0]]
         )
 
-        set_message_history(truncated_history)
+        agent.set_message_history(truncated_history)
         emit_success(
             f"Truncated message history from {len(history)} to {len(truncated_history)} messages (keeping system message and {n - 1} most recent)"
         )
@@ -697,8 +761,30 @@ def handle_command(command: str):
         # Signal to the main app that we want to exit
         # The actual exit handling is done in main.py
         return True
+
+    # Try plugin-provided custom commands before unknown warning
     if command.startswith("/"):
+        # Extract command name without leading slash and arguments intact
         name = command[1:].split()[0] if len(command) > 1 else ""
+        try:
+            from code_puppy import callbacks
+
+            results = callbacks.on_custom_command(command=command, name=name)
+            # Iterate through callback results; treat str as handled (no model run)
+            for res in results:
+                if res is True:
+                    return True
+                if isinstance(res, str):
+                    # Display returned text to the user and treat as handled
+                    try:
+                        emit_info(res)
+                    except Exception:
+                        pass
+                    return True
+        except Exception as e:
+            # Log via emit_error but do not block default handling
+            emit_warning(f"Custom command hook error: {e}")
+
         if name:
             emit_warning(
                 f"Unknown command: {command}\n[dim]Type /help for options.[/dim]"
