@@ -94,7 +94,7 @@ def get_model_context_length() -> int:
         from code_puppy.model_factory import ModelFactory
 
         model_configs = ModelFactory.load_config()
-        model_name = get_model_name()
+        model_name = get_global_model_name()
 
         # Get context length from model config
         model_config = model_configs.get(model_name, {})
@@ -162,8 +162,6 @@ def load_mcp_server_configs():
         return {}
 
 
-
-
 def _default_model_from_models_json():
     """Attempt to load the first model name from models.json.
 
@@ -298,14 +296,18 @@ def _validate_model_exists(model_name: str) -> bool:
 
 def clear_model_cache():
     """Clear the model validation cache. Call this when models.json changes."""
-    global _model_validation_cache, _default_model_cache, _default_vision_model_cache, _default_vqa_model_cache
+    global \
+        _model_validation_cache, \
+        _default_model_cache, \
+        _default_vision_model_cache, \
+        _default_vqa_model_cache
     _model_validation_cache.clear()
     _default_model_cache = None
     _default_vision_model_cache = None
     _default_vqa_model_cache = None
 
 
-def get_model_name():
+def get_global_model_name():
     """Return a valid model name for Code Puppy to use.
 
     1. Look at ``model`` in *puppy.cfg*.
