@@ -12,27 +12,18 @@ from textual.events import Resize
 from textual.reactive import reactive
 from textual.widgets import Footer, ListView
 
-from code_puppy.agent import get_code_generation_agent, get_custom_usage_limits
-from code_puppy.agents.runtime_manager import get_runtime_agent_manager
+# message_history_accumulator and prune_interrupted_tool_calls have been moved to BaseAgent class
+from code_puppy.agents.agent_manager import get_current_agent
 from code_puppy.command_line.command_handler import handle_command
 from code_puppy.config import (
-    get_model_name,
+    get_global_model_name,
     get_puppy_name,
     initialize_command_history_file,
     save_command_to_history,
 )
-from code_puppy.message_history_processor import (
-    message_history_accumulator,
-    prune_interrupted_tool_calls,
-)
 
 # Import our message queue system
 from code_puppy.messaging import TUIRenderer, get_global_queue
-from code_puppy.state_management import (
-    clear_message_history,
-    get_message_history,
-    set_message_history,
-)
 from code_puppy.tui.components import (
     ChatView,
     CustomTextArea,
@@ -41,8 +32,6 @@ from code_puppy.tui.components import (
     StatusBar,
 )
 
-from .. import state_management
-
 # Import shared message classes
 from .messages import CommandSelected, HistoryEntrySelected
 from .models import ChatMessage, MessageType
@@ -112,9 +101,9 @@ class CodePuppyTUI(App):
     def _on_agent_reload(self, agent_id: str, agent_name: str) -> None:
         """Callback for when agent is reloaded/changed."""
         # Get the updated agent configuration
-        from code_puppy.agents.agent_manager import get_current_agent_config
+        from code_puppy.agents.agent_manager import get_current_agent
 
-        current_agent_config = get_current_agent_config()
+        current_agent_config = get_current_agent()
         new_agent_display = (
             current_agent_config.display_name if current_agent_config else "code-puppy"
         )
@@ -127,7 +116,6 @@ class CodePuppyTUI(App):
 
     def __init__(self, initial_command: str = None, **kwargs):
         super().__init__(**kwargs)
-        self.agent_manager = None
         self._current_worker = None
         self.initial_command = initial_command
 
@@ -159,13 +147,13 @@ class CodePuppyTUI(App):
         register_callback("agent_reload", self._on_agent_reload)
 
         # Load configuration
-        self.current_model = get_model_name()
+        self.current_model = get_global_model_name()
         self.puppy_name = get_puppy_name()
 
         # Get current agent information
-        from code_puppy.agents.agent_manager import get_current_agent_config
+        from code_puppy.agents.agent_manager import get_current_agent
 
-        current_agent_config = get_current_agent_config()
+        current_agent_config = get_current_agent()
         self.current_agent = (
             current_agent_config.display_name if current_agent_config else "code-puppy"
         )
@@ -174,8 +162,6 @@ class CodePuppyTUI(App):
         self._update_title()
 
         # Use runtime manager to ensure we always have the current agent
-        self.agent_manager = get_runtime_agent_manager()
-
         # Update status bar
         status_bar = self.query_one(StatusBar)
         status_bar.current_model = self.current_model
@@ -187,12 +173,6 @@ class CodePuppyTUI(App):
             "Welcome to Code Puppy 🐶!\n💨 YOLO mode is enabled in TUI: commands will execute without confirmation."
         )
 
-        # Get current agent and display info
-        get_code_generation_agent()
-        self.add_system_message(
-            f"🐕 Loaded agent '{self.puppy_name}' with model '{self.current_model}'"
-        )
-
         # Start the message renderer EARLY to catch startup messages
         # Using call_after_refresh to start it as soon as possible after mount
         self.call_after_refresh(self.start_message_renderer_sync)
@@ -430,11 +410,6 @@ class CodePuppyTUI(App):
                 else:
                     # Only cancel the agent task if NO processes were killed
                     self._current_worker.cancel()
-                    state_management.set_message_history(
-                        prune_interrupted_tool_calls(
-                            state_management.get_message_history()
-                        )
-                    )
                     self.add_system_message("⚠️  Processing cancelled by user")
                     # Stop spinner and clear state only when agent is actually cancelled
                     self._current_worker = None
@@ -491,55 +466,42 @@ class CodePuppyTUI(App):
                 return
 
             # Process with agent
-            if self.agent_manager:
+            try:
+                self.update_agent_progress("Processing", 25)
+
+                # Use agent_manager's run_with_mcp to handle MCP servers properly
                 try:
-                    self.update_agent_progress("Processing", 25)
-
-                    # Use agent_manager's run_with_mcp to handle MCP servers properly
-                    try:
-                        self.update_agent_progress("Processing", 50)
-                        result = await self.agent_manager.run_with_mcp(
-                            message,
-                            message_history=get_message_history(),
-                            usage_limits=get_custom_usage_limits(),
-                        )
-
-                        if not result or not hasattr(result, "output"):
-                            self.add_error_message("Invalid response format from agent")
-                            return
-
-                        self.update_agent_progress("Processing", 75)
-                        agent_response = result.output
-                        self.add_agent_message(agent_response)
-
-                        # Update message history
-                        new_msgs = result.new_messages()
-                        message_history_accumulator(new_msgs)
-
-                        # Refresh history display to show new interaction
-                        self.refresh_history_display()
-
-                    except Exception as eg:
-                        # Handle TaskGroup and other exceptions
-                        # BaseExceptionGroup is only available in Python 3.11+
-                        if hasattr(eg, "exceptions"):
-                            # Handle TaskGroup exceptions specifically (Python 3.11+)
-                            for e in eg.exceptions:
-                                self.add_error_message(f"MCP/Agent error: {str(e)}")
-                        else:
-                            # Handle regular exceptions
-                            self.add_error_message(f"MCP/Agent error: {str(eg)}")
-                    finally:
-                        set_message_history(
-                            prune_interrupted_tool_calls(get_message_history())
-                        )
-                except Exception as agent_error:
-                    # Handle any other errors in agent processing
-                    self.add_error_message(
-                        f"Agent processing failed: {str(agent_error)}"
+                    agent = get_current_agent()
+                    self.update_agent_progress("Processing", 50)
+                    result = await agent.run_with_mcp(
+                        message,
                     )
-            else:
-                self.add_error_message("Agent manager not initialized")
+
+                    if not result or not hasattr(result, "output"):
+                        self.add_error_message("Invalid response format from agent")
+                        return
+
+                    self.update_agent_progress("Processing", 75)
+                    agent_response = result.output
+                    self.add_agent_message(agent_response)
+                    # Refresh history display to show new interaction
+                    self.refresh_history_display()
+
+                except Exception as eg:
+                    # Handle TaskGroup and other exceptions
+                    # BaseExceptionGroup is only available in Python 3.11+
+                    if hasattr(eg, "exceptions"):
+                        # Handle TaskGroup exceptions specifically (Python 3.11+)
+                        for e in eg.exceptions:
+                            self.add_error_message(f"MCP/Agent error: {str(e)}")
+                    else:
+                        # Handle regular exceptions
+                        self.add_error_message(f"MCP/Agent error: {str(eg)}")
+                finally:
+                    pass
+            except Exception as agent_error:
+                # Handle any other errors in agent processing
+                self.add_error_message(f"Agent processing failed: {str(agent_error)}")
 
         except Exception as e:
             self.add_error_message(f"Error processing message: {str(e)}")
@@ -553,7 +515,8 @@ class CodePuppyTUI(App):
         """Clear the chat history."""
         chat_view = self.query_one("#chat-view", ChatView)
         chat_view.clear_messages()
-        clear_message_history()
+        agent = get_current_agent()
+        agent.clear_message_history()
         self.add_system_message("Chat history cleared")
 
     def action_show_help(self) -> None:
@@ -652,16 +615,21 @@ class CodePuppyTUI(App):
         def handle_settings_result(result):
             if result and result.get("success"):
                 # Update reactive variables
-                from code_puppy.config import get_model_name, get_puppy_name
+                from code_puppy.config import get_global_model_name, get_puppy_name
 
                 self.puppy_name = get_puppy_name()
 
                 # Handle model change if needed
                 if result.get("model_changed"):
-                    new_model = get_model_name()
+                    new_model = get_global_model_name()
                     self.current_model = new_model
-                    # Reinitialize agent with new model
-                    self.agent_manager.reload_agent()
+                    try:
+                        current_agent = get_current_agent()
+                        current_agent.reload_code_generation_agent()
+                    except Exception as reload_error:
+                        self.add_error_message(
+                            f"Failed to reload agent after model change: {reload_error}"
+                        )
 
                 # Update status bar
                 status_bar = self.query_one(StatusBar)
